from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponse, HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from .models import PAC
from categories.models import Category
from products.models import Product
from django.core.paginator import Paginator
from decimal import Decimal, InvalidOperation
from django.contrib import messages
from django.db import IntegrityError

@login_required
def pacs(request):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            pacs_list = PAC.objects.all()
            paginator = Paginator(pacs_list, 10)
            page_number = request.GET.get('page')
            page_obj = paginator.get_page(page_number)

            if request.method == 'POST':
                product_id = request.POST.get('product')
                category_id = request.POST.get('category')
                pac_value_str = request.POST.get('pac_value')

                try:
                    pac_value = Decimal(pac_value_str)
                except InvalidOperation:
                    messages.error(request, 'Invalid PAC value. Please enter a valid decimal number.')
                    products = Product.objects.all()
                    categories = Category.objects.all()
                    return render(request, "inventory/pac/add_pac.html", {'products': products, 'categories': categories})

                pac = PAC(product_id=product_id, category_id=category_id, pac_value=pac_value, added_by=request.user)
                pac.save()
                messages.success(request, 'PAC added successfully.')
                return redirect("inventory:pacs")

            products = Product.objects.all()
            categories = Category.objects.all()
            return render(request, "inventory/pac/pacs.html", {'products': products, 'categories': categories, 'page_obj': page_obj})
        except IntegrityError as e:
            messages.error(request, f"Data integrity error: {str(e)}")
            return redirect("inventory:pacs")
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
            return redirect("inventory:pacs")
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def edit_pac(request, pac_id):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            pac = get_object_or_404(PAC, id=pac_id)
            if request.method == 'POST':
                pac.product_id = request.POST.get('product')
                pac.category_id = request.POST.get('category')
                pac_value_str = request.POST.get('pac_value')
                try:
                    pac.pac_value = Decimal(pac_value_str)
                except InvalidOperation:
                    messages.error(request, 'Invalid PAC value. Please enter a valid decimal number.')
                    products = Product.objects.all()
                    categories = Category.objects.all()
                    return render(request, "inventory/pac/edit_pac.html", {'pac': pac, 'products': products, 'categories': categories})
                pac.save()
                messages.success(request, 'PAC updated successfully.')
                return redirect("inventory:pacs")
            products = Product.objects.all()
            categories = Category.objects.all()
            return render(request, "inventory/pac/edit_pac.html", {'pac': pac, 'products': products, 'categories': categories})
        except IntegrityError as e:
            messages.error(request, f"Data integrity error: {str(e)}")
            return redirect("inventory:pacs")
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
            return redirect("inventory:pacs")
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def delete_pac(request, pac_id):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            pac = get_object_or_404(PAC, id=pac_id)
            pac_value = pac.pac_value
            pac.delete()
            messages.success(request, f"PAC with value '{pac_value}' has been deleted successfully.")
        except Exception as e:
            messages.error(request, f"An error occurred while deleting the PAC: {str(e)}")
        return redirect("inventory:pacs")
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")